#ifndef IP_ENHANCE_DCT2D_INC
#define IP_ENHANCE_DCT2D_INC

#include "ipCore.h"

#include "ipDCT2D.h"

namespace Torch {

/** This class is designed to compute enhanced DCT
 
    This class computes 2D DCT with Image Enhancement in the compress domain as described in the paper below

    \begin{verbatim}
       Jinshan Tang, Eli Peli and Scott Acton, 
       "Image Enhancement Using a Contrast Measure in the Compressed Domain",
       IEEE Signal Processing Letters, Vol 10, No 10, October 2003
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @author Fabien Cardinaux (cardinau@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipEnhanceDCT2D : public ipDCT2D
{
   	// number of bands
   	int n_band;

	// internal variables
	real *d;
	real *E;
	real *H;
public:
	/// enhancement factor
	real lambda;
	
	//-----
	
	/** constructor
	
	    @param block_size_ is the input size of image to process
	    @param coding_ is the coding format of the image to process
            @param dct_dim_ is the number of DCT coefficient to keep
	    @param lambda_ enhancement factor
	*/
	ipEnhanceDCT2D(int block_size_, const char *coding_, int dct_dim_, real lambda_ = 1.5);

	///
    	virtual ~ipEnhanceDCT2D();

	///
    	virtual void process(Sequence *seq_in);
};

}

#endif
