#ifndef IPFLIP_INC
#define IPFLIP_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to flip an image

    \begin{verbatim}

         +---+          +--------------+         +---+
         |AAA|	        |              |         |BBB|
	 |XXX|   ---->  |    ipFlip    | ---->   |XXX|
         |BBB|          |              |         |AAA|
         +---+          +--------------+         +---+

    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipFlip : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipFlip(int width_, int height_, const char *coding_);

	///
	virtual ~ipFlip();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
