#ifndef IP_IDENTITY_INC
#define IP_IDENTITY_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to copy an image

    \begin{verbatim}

         +---+          +--------------+         +---+
         |XXX|	        |              |         |XXX|
	 |XXX|   ---->  |  ipIdentity  | ---->   |XXX|
         |XXX|          |              |         |XXX|
         +---+          +--------------+         +---+

	 image                                   image 
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipIdentity : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipIdentity(int width_, int height_, const char *coding_);

	///
	virtual ~ipIdentity();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
