#ifndef IPINTEGRALIMAGE_INC
#define IPINTEGRALIMAGE_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to compute the integral image of an image

    \begin{equation}
    II(x,y) = \sum_{i=1}^{x-1} \sum_{j=1}^{y-1} I(i,j)
    \end{equation}
    
    \begin{verbatim}

         +---+          +--------------+         +---+
         |XXX|	        |              |         |XXX|
	 |XXX|   ---->  |  ipIdentity  | ---->   |XXX|
         |XXX|          |              |         |XXX|
         +---+          +--------------+         +---+

	 image                                integral image 

    \end{verbatim}
    
    @author Yann Rodriguez (rodrig@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)

    @version 2.0
    \Date
    @since 2.0
*/
class ipIntegralImage : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// compute integral image square
	bool square;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param square_ is true if you want to compute the integral image square
	*/
    	ipIntegralImage(int width_, int height_, const char *coding_, bool square_=false);

	///
	virtual ~ipIntegralImage();

	///
	virtual void process(Sequence *seq_in);
	
};

// This function computes the sum of pixels of a subwindow using the integral image
real iisum_check_limit(real *ii, int width, int height, int x, int y, int w, int h);

/// This function computes the sum of pixels of a subwindow using the integral image
real iisum(real *ii, int width, int height, int x, int y, int w, int h);

}

#endif
