#include "ipSmoothGaussian3.h"

namespace Torch {

void ipSmoothGaussian3::init_(void)
{
  	int i, j, i_1, j_1;
  	int index;

  	real sum = 0.0;
  
  	for(i = 0 ; i < 3 ; i++) 
    		for(j = 0 ; j < 3 ; j++)
    		{
      			index = j + i * 3;

      			i_1 = i - 1;
      			j_1 = j - 1;

      			kernel[index] = exp(-(((float) (i_1 * i_1 + j_1 * j_1)) / sigma));
      			sum += kernel[index];
    		}
  
  	for(int i = 0; i < 9 ; i++) kernel[i] /= sum;
}
	
ipSmoothGaussian3::ipSmoothGaussian3(int width_, int height_, char *coding_, real sigma_) : ipCore(coding_)
{
	width = width_;
        height = height_;
	sigma = sigma_;

	seq_out = new(allocator) Sequence(1, width_*height_*n_planes);
	
	for(int i=0; i<seq_out->frame_size; i++)
		seq_out->frames[0][i] = 0.0;

   	kernel = new real [9];

	init_();
}

void ipSmoothGaussian3::process(Sequence *seq_in)
{
	switch(coding_id)
	{
	case 0:
      		for(int i = 0 ; i < height ; i++)
	      	{
        		for(int j = 0 ; j < width ; j++)
			{
		  		real sum = 0.0;
			 
		  		for(int n = 0 ; n < 3 ; n++)
	    				for(int m = 0 ; m < 3 ; m++)
	    				{
		      				int ii = i;
		      				int jj = j;
	      					if((i == 0) && (n == 0)) ii += 1;				 
	      					if((j == 0) && (m == 0)) jj += 1;					 
	      					if((i == (height - 1)) && (n == (3 - 1))) ii -= 1;
	      					if((j == (width - 1)) && (m == (3 - 1))) jj -= 1;
	      
		      				sum += kernel[m + n * 3] * seq_in->frames[0][jj + m - 1 + (ii + n - 1) * width];
		    			}
			 
	            		seq_out->frames[0][j + i * width] = sum;
		 	}
	      	}
		break;
	case 1:
      		for(int i = 0 ; i < height ; i++)
	      	{
		   	int base_in = i * width;
			
        		for(int j = 0 ; j < width ; j++)
			{
		  		real sum0 = 0.0;
		  		real sum1 = 0.0;
		  		real sum2 = 0.0;
				int base;

		  		for(int n = 0 ; n < 3 ; n++)
	    				for(int m = 0 ; m < 3 ; m++)
	    				{
		      				int ii = i;
		      				int jj = j;
	      					if((i == 0) && (n == 0)) ii += 1;				 
	      					if((j == 0) && (m == 0)) jj += 1;					 
	      					if((i == (height - 1)) && (n == (3 - 1))) ii -= 1;
	      					if((j == (width - 1)) && (m == (3 - 1))) jj -= 1;
	      
						base = 3 * (jj + m - 1 + (ii + n - 1) * width);
						
		      				sum0 += kernel[m + n * 3] * seq_in->frames[0][base];
		      				sum1 += kernel[m + n * 3] * seq_in->frames[0][base+1];
		      				sum2 += kernel[m + n * 3] * seq_in->frames[0][base+2];
		    			}
			 
				base = 3 * (base_in + j);
				
	            		seq_out->frames[0][base] = sum0;
	            		seq_out->frames[0][base+1] = sum1;
	            		seq_out->frames[0][base+2] = sum2;
		 	}
	      	}
		break;
	case 2:
	case 3:
		message("ipSmoothGaussian3::process() not implemented for YUV and float format");
		break;
	}
}

ipSmoothGaussian3::~ipSmoothGaussian3()
{
   	delete [] kernel;
}

}

