#ifndef PGM_DISK_X_FILE_INC
#define PGM_DISK_X_FILE_INC

#include "ImageDiskXFile.h"

namespace Torch {

/** This class is designed to handle a PGM image on the disk
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class pgmDiskXFile : public ImageDiskXFile
{
public:
   	/**@name constructor and destructor */
        //@{
	/** Makes a ImageDiskXFile from an filename.
	
	    @param file_name is the filename of the image
	    @param open_flags is the opening mode of the file
	*/
	pgmDiskXFile(const char *file_name, const char *open_flags);

	/// Destructor
    	virtual ~pgmDiskXFile();
	//@}
	
   	/**@name read/write image header and image pixmap */
        //@{
	/** read the header from the file.
	
	    @param width_ is the width of the image read in the header
	    @param height_ is the height of the image read in the header
	*/
	virtual bool readHeader(int *width_, int *height_);

	/// read the #pixmap# from the file
	virtual int readPixmap();

	/** write the header from the file.
	
	    @param width_ is the width of the image written in the header
	    @param height_ is the height of the image written in the header
	*/
	virtual bool writeHeader(int width_, int height_);

	/// write the #pixmap_# into the file
	virtual int writePixmap(unsigned char *pixmap_);
	//@}
};

}

#endif
