#include "xtprobeImageDiskXFile.h"

#include "pgmDiskXFile.h"
#include "ppmDiskXFile.h"
#include "jpegDiskXFile.h"
#include "gifDiskXFile.h"
#include "tiffDiskXFile.h"

namespace Torch {

static int n_imageformat = 12;

struct imageformat_
{
   	char *extension;
	char *coding;
	int n_planes;
};

static struct imageformat_ imageformat[] = {	{"ppm", "rgb", 3},
	                                	{"jpg", "rgb", 3},
                	                	{"jpeg", "rgb", 3},
                        	        	{"JPG", "rgb", 3},
                                		{"JPEG", "rgb", 3},
	                                	{"gif", "rgb", 3},
        	                        	{"GIF", "rgb", 3},
                	                	{"pgm", "gray", 1},
                	                	{"TIF", "rgb", 3},
                	                	{"TIFF", "rgb", 3},
                	                	{"tif", "rgb", 3},
                	                	{"tiff", "rgb", 3}};

int FindImageFormat(const char *filename);

xtprobeImageDiskXFile::xtprobeImageDiskXFile(const char *file_name, const char *open_flags) : ImageDiskXFile()
{
	ixfile = NULL;
	n_planes = 0;
	
	int i = FindImageFormat(file_name);
	
	if(i != -1)
	{
        	coding = imageformat[i].coding;
		n_planes = imageformat[i].n_planes;

		//print("Probe format: %s (%d) %d\n", coding, i, n_planes);
		
		switch(i)
		{
		case 0:
		   	ixfile = new ppmDiskXFile(file_name, open_flags);
		   	break;
		case 1:
		case 2:
		case 3:
		case 4:
		   	ixfile = new jpegDiskXFile(file_name, open_flags);
			break;
		case 5:
		case 6:
		   	ixfile = new gifDiskXFile(file_name, open_flags);
			break;
		case 7:
		   	ixfile = new pgmDiskXFile(file_name, open_flags);
			break;
		case 8:
		case 9:
		case 10:
		case 11:
		   	ixfile = new tiffDiskXFile(file_name, open_flags);
			break;
		}
	}
	else warning("xtprobeImageDiskXFile::xtprobeImageDiskXFile(%s) Impossible to probe image format from extension.", file_name);
}

bool xtprobeImageDiskXFile::readHeader(int *width_, int *height_)
{	
  	if(ixfile == NULL)
	{
	   	warning("xtprobeImageDiskXFile::readHeader() no loader available.");
		
	   	return false;
	}
	
	ixfile->setBOption("verbose", verbose);
	
   	if(ixfile->readHeader(&width, &height) == false)
	{
		*width_ = 0;
		*height_ = 0;
	   	
	   	return false;
	}

	*width_ = width;
	*height_ = height;
	
	return true;
}

int xtprobeImageDiskXFile::readPixmap()
{
  	if(ixfile == NULL)
	{
	   	warning("xtprobeImageDiskXFile::readPixmap() no loader available.");
		
	   	return -1;
	}

	ixfile->readPixmap();

	pixmap = ixfile->pixmap;
	
	return 0;
}

bool xtprobeImageDiskXFile::writeHeader(int width_, int height_)
{
   	width = width_;
	height = height;

	return true;
}

int xtprobeImageDiskXFile::writePixmap(unsigned char *pixmap_)
{
	return 0;
}

xtprobeImageDiskXFile::~xtprobeImageDiskXFile()
{
   	if(ixfile != NULL) delete ixfile;
}

int FindImageFormat(const char *filename)
{
	bool verbose = false;
	char *extension;
	char *str = (char *) filename;

	/*
	if(str[0] == '.')
	{
		str++;
		if(str[0] == '.') str ++;
	}
	*/

	//extension = (char *) rindex(filename, '.');
	extension = strrchr(str, '.');
	extension++;

	if(verbose == true) 
	{
		print("Checking file format of %s (%s):\n", filename, extension);
		print(" + Trying");
	}

 	for(int i = 0 ; i < n_imageformat ; i++)
        {
        	if(verbose == true) print(" %s", imageformat[i].extension);

                if(strcmp(extension, imageformat[i].extension) == 0)
                {
                	if(verbose == true) print(" [OK]\n");
                         
                        return i;
                } else if(verbose == true) print("...");
        }
  
	if(verbose == true) print("\n");

	return -1;  
}


}
