const char *help = "\
progname: filtercolorcube.cc\n\
code2html: This program reads a ppm image and filter it using a colorcube.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "ipColorCube.h"
#include "DiskXFile.h"
#include "xtprobeImageDiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	char *colorcube_filename;
	bool verbose;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addSCmdArg("colorcube filename", &colorcube_filename, "colorcube filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);


	Image *image_in = NULL;
	Image *image_out = NULL;

	image_in = new ImageRgb();
	image_in->setBOption("verbose", verbose);
	
	ImageDiskXFile *image_loader = new xtprobeImageDiskXFile(image_filename, "r");
	image_in->loadImageXFile(image_loader);
	delete image_loader;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image_in->width);
		print("   height = %d\n", image_in->height);
		print("   format = %s (%d)\n", image_in->coding, image_in->n_planes);
	}

	ipCore *imachine = NULL;

	imachine = new ipColorCube(image_in->width, image_in->height, "rgb");
	imachine->setBOption("verbose", verbose);
	imachine->load(colorcube_filename);
	imachine->process(image_in);
	
	image_out = new ImageRgb();
	image_out->setBOption("verbose", verbose);

	image_out->copyFrom(image_in->width, image_in->height, imachine->seq_out->frames[0], "rgb");
	
	DiskXFile *image_file = new DiskXFile("filter.ppm", "w");
	image_out->saveXFile(image_file);
	delete image_file;

	delete imachine;
	delete image_out;
	delete image_in;

	return(0);
}
