#ifndef EIGEN_MACHINE_INC
#define EIGEN_MACHINE_INC

#include "Machine.h"
#include "XFile.h"
#include "PreProcessing.h"

namespace Torch {

/** This class is designed to project vectors of data into an eigenvector matrix 

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class EigenMachine : public Machine
{  
public:
   	/// input dimension
   	int n_inputs;

	/// output dimension
	int n_outputs;
	
	/// eigenvalues
    	real *eigenvalues;

	/// eigenvectors
    	real **eigenvectors;

	/// variance
	real variance;

	/// verbosity
	int verbose_level;

	//-----

	///
	EigenMachine(int n_inputs_);	

	///
	virtual ~EigenMachine();

	///
	void init();

	/// computes the projection of the input onto #eigenvectors#
	virtual void forward(Sequence *inputs);

	/// load the parameters of the eigenmachine
	virtual void loadXFile(XFile *file);

	/// save the parameters of the eigenmachine
	virtual void saveXFile(XFile *file);

	/// load the parameters of the eigenmachine with preprocessing
	void load(const char *filename, PreProcessing *preprocess_norm=NULL);

	/// save the parameters of the eigenmachine with preprocessing
	void save(const char *filename, PreProcessing *preprocess_norm=NULL);
};

}

#endif
