#include "LDAMachine.h"

#include "DiskXFile.h"

namespace Torch {

LDAMachine::LDAMachine(int n_inputs_, bool norm_d_mean_) : EigenMachine(n_inputs_)
{
   	//
	d_m_mean_ = NULL;
	d_m_mean_ = (real *) allocator->alloc(sizeof(real) * n_inputs);
	
	norm_d_mean = norm_d_mean_;
}

void LDAMachine::forward(Sequence *inputs)
{
   	if(verbose_level >= 3) print("LDAMachine::forward() %d->%d ...\n", n_inputs, n_outputs);
	
	for(int i = 0 ; i < n_outputs ; i++)
	{
	   	outputs->frames[0][i] = 0.0;

       		for(int j = 0 ; j < n_inputs ; j++)
           		outputs->frames[0][i] += eigenvectors[j][i] * inputs->frames[0][j];

		if(norm_d_mean) outputs->frames[0][i] -= d_m_mean_[i];
	}
}

void LDAMachine::loadXFile(XFile *file)
{
   	if(verbose_level >= 2) print("LDAMachine::loadXFile() ...\n");

   	EigenMachine::loadXFile(file);
	file->taggedRead(d_m_mean_, sizeof(real), n_inputs, "D_MEANS");
}

void LDAMachine::saveXFile(XFile *file)
{
   	if(verbose_level >= 2) print("LDAMachine::saveXFile() ...\n");

   	EigenMachine::saveXFile(file);
	file->taggedWrite(d_m_mean_, sizeof(real), n_inputs, "D_MEANS");
}

LDAMachine::~LDAMachine()
{
}

}
