#ifndef MY_MEAN_VAR_NORM_INC
#define MY_MEAN_VAR_NORM_INC

#include "PreProcessing.h"
#include "DataSet.h"

namespace Torch {

/** This class is designed to compute mean and variance for normalization

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class MyMeanVarNorm : public PreProcessing
{
private:
    void normalizeSequence(Sequence *sequence, real *mean, real *stdv);

public:
    /// input frame size
    int n_inputs;

    /// target frame size
    int n_targets;

    /// inputs means array
    real *inputs_mean;

    /// targets means array
    real *targets_mean;

    /// inputs standard deviations array
    real *inputs_stdv;

    /// targets standard deviations array
    real *targets_stdv;

    ///
    MyMeanVarNorm(int n_inputs_, int n_targets_, bool norm_inputs=true, bool norm_targets=false);

    ///
    virtual ~MyMeanVarNorm();

    ///
    virtual void preProcessInputs(Sequence *inputs);

    ///
    virtual void preProcessTargets(Sequence *targets);

    /// load means and standard deviations
    virtual void loadXFile(XFile *file);

    /// save means and standard deviations
    virtual void saveXFile(XFile *file);

    ///
    void normalize(int n_total_frames);
};

}

#endif
