#include "PatternMerger.h"

namespace Torch {

PatternMerger::PatternMerger()
{
	addBOption("verbose", &verbose, false, "verbose");

	n_patterns = 0;
}

PatternMerger::~PatternMerger()
{
}
	

meanRectPatternMerger::meanRectPatternMerger()
{
}

void meanRectPatternMerger::reset()
{
	if(verbose) print(" * meanRectPatternMerger::reset()\n");

	//
	n_patterns = 0;

	//
	m_cx = 0.0;
	m_cy = 0.0;
	m_w = 0.0;
	m_h = 0.0;
	m_scale = 0.0;
	m_view = 0.0;
	m_confidence = 0.0;
	max_confidence = -1000.0;
}
	
void meanRectPatternMerger::add(Pattern *p_)
{
	// cumulate
	m_cx += p_->x + (real)p_->w / 2.0;
	m_cy += p_->y + (real)p_->h / 2.0;
	m_w += (real) p_->w;
	m_h += (real) p_->h;
	m_scale += p_->scale;
	m_view += p_->view;
	m_confidence += p_->confidence;

	if(p_->confidence > max_confidence) max_confidence = p_->confidence;

	n_patterns++;	

	if(verbose) print(" + meanRectPatternMerger::add(%d): %2.3f max=%2.3f\n", n_patterns, p_->confidence, max_confidence);
}
	
bool meanRectPatternMerger::merge(Pattern *p_)
{
	// average
   	m_cx /= n_patterns;
   	m_cy /= n_patterns;
	m_w /= n_patterns;
	m_h /= n_patterns;
	m_scale /= n_patterns;
	m_view /= n_patterns;
	m_confidence /= n_patterns;

	// 
	p_->w = (int) m_w;
	p_->h = (int) m_h;
	p_->x = (int) (m_cx - m_w/2.0);
	p_->y = (int) (m_cy - m_h/2.0);
	p_->scale = m_scale;
	p_->view = (int) m_view;
	p_->confidence = m_confidence;

	if(verbose) print(" > meanRectPatternMerger::merge(): %2.3f\n", m_confidence);

	return true;
}

meanRectPatternMerger::~meanRectPatternMerger()
{
}

}
