#include "WindowScan.h"

namespace Torch {
  
#define IS_NEAR(var, value, delta) ((var >= (value - delta)) && (var <= (value + delta)))

WindowScan::WindowScan(	int width_image_, int height_image_, 
			int real_width_image_, int real_height_image_, 
			int width_pattern_, int height_pattern_, 
			int real_width_pattern_, int real_height_pattern_, 
			real stepx_factor_, real stepy_factor_, 
			ipSubWindow *ip_sw_,
			PatternCollector *patterncollector_, 
			bool verbose_)
{
	verbose = verbose_;
	
	width_image = width_image_;
	height_image = height_image_;

	real_width_image = real_width_image_;
	real_height_image = real_height_image_;

	width_pattern = width_pattern_;
	height_pattern = height_pattern_;
	
	real_width_pattern = real_width_pattern_;
	real_height_pattern = real_height_pattern_;
	
	stepx = FixI((real) width_pattern * stepx_factor_);
	stepy = FixI((real) height_pattern * stepy_factor_);

	ip_sw = ip_sw_;
	
	patterncollector = patterncollector_;
	
	n_subwindows = 0;
	n_subwindows_processed = 0;
	n_pruning_reject = 0;
	n_mean_reject = 0;
	n_stdv_reject = 0;
	
	//mean stdv
	use_mean_stdv = false;
        minMean = -1;
        maxMean = -1;
        minStdv = -1;
        maxStdv = -1;
        integralimage = NULL;
        integralimage2 = NULL;
        one_by_size_pattern = -1;

	// pruning
	use_pruning = false;
	pruning_ii = NULL;
	pruning_min_threshold = -1;
	pruning_max_threshold = -1;
}

void WindowScan::setMeanStdv(real minMean_, real maxMean_, real minStdv_, real maxStdv_, real *integralimage_, real *integralimage2_)
{
	minMean = minMean_;
        maxMean = maxMean_;
        minStdv = minStdv_;
        maxStdv = maxStdv_;
	
	integralimage = integralimage_;
	integralimage2 = integralimage2_;

	if(integralimage != NULL && integralimage2 != NULL)
	{
		// for faster computation --------
		int size_pattern = real_width_pattern * real_height_pattern;
		one_by_size_pattern = 1.0/size_pattern;
	
		minMean *= size_pattern;
		maxMean *= size_pattern;

		minStdv *= minStdv;
		maxStdv *= maxStdv;
		minStdv *= size_pattern;
		maxStdv *= size_pattern;
		//---------------------------------
	
		use_mean_stdv = true;
	}
}
	
void WindowScan::setPruningII(real pruning_min_threshold_, real pruning_max_threshold_, real *pruning_ii_)
{
	pruning_ii = pruning_ii_;

	if(pruning_ii != NULL)
	{
		pruning_min_threshold = pruning_min_threshold_;
		pruning_max_threshold = pruning_max_threshold_;
			
		int size_pattern = real_width_pattern * real_height_pattern;
		pruning_min_threshold *= size_pattern;
		pruning_max_threshold *= size_pattern;

		use_pruning = true;
	}
}

WindowScan::~WindowScan()
{
}
	
}
