#include "FaceModel.h"

namespace Torch {
	
FaceModel::FaceModel(int postype_) : ShapeModel(5)
{
	// default loading format
	postype = postype_;

	//
	r_eye_idx = -1;
	l_eye_idx = -1;
	nosetip_idx = -1;
	chin_idx = -1;
}

double FaceModel::getAngle()
{
   	if(r_eye_idx != -1 && l_eye_idx != -1)
	{   
		double angle = -atan2(getReye().y - getLeye().y, getReye().x - getLeye().x);

		return angle;
	}
	else return 0.0;
}
	
sPoint2D FaceModel::getCenter()
{
	sPoint2D p;

	p.x = -1;
	p.y = -1;

	return p;
}


sPoint2D FaceModel::getReye()
{
   	if(r_eye_idx == -1) error("FaceModel::getReye() this landmark does not exist");

	return ldm_points[r_eye_idx];
}

sPoint2D FaceModel::getLeye()
{
   	if(l_eye_idx == -1) error("FaceModel::getLeye() this landmark does not exist");

	return ldm_points[l_eye_idx];
}
	
sPoint2D FaceModel::getNosetip()
{
   	if(nosetip_idx == -1) error("FaceModel::getNosetip() this landmark does not exist");

	return ldm_points[nosetip_idx];
}
	
sPoint2D FaceModel::getChin()
{
   	if(chin_idx == -1) error("FaceModel::getChin() this landmark does not exist");

	return ldm_points[chin_idx];
}

void FaceModel::loadXFile(XFile *file)
{
   	n_ldm_points = 0;
	r_eye_idx = -1;
	l_eye_idx = -1;
	nosetip_idx = -1;
	chin_idx = -1;
	
	if(postype == 1)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 2 * sizeof(sPoint2D));

		if(verbose) message("reading eye centers pos format.");
			
		// IDIAP format (eyes center)
		real x, y;
		
		// left eye
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;
		
		// right eye
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		l_eye_idx = 0;
		r_eye_idx = 1;

		if(verbose) message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
	}
	else if(postype == 2)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 10 * sizeof(sPoint2D));

		if(verbose) message("reading Banca pos format.");

		// BANCA format
		real x, y;
		
		for(int i = 0 ; i < 10 ; i++)
		{
			file->scanf("%f", &x);
			file->scanf("%f", &y);
			ldm_points[n_ldm_points].x = x;
			ldm_points[n_ldm_points++].y = y;
		}
		
		l_eye_idx = 1;
		r_eye_idx = 4;

		if(verbose) message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
	}
	else if(postype == 3)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 2 * sizeof(sPoint2D));

		if(verbose) message("reading eye corners pos format.");

		// IDIAP format (eyes corner)
		real l_eye_l_x, l_eye_l_y;
		file->scanf("%f", &l_eye_l_x);
		file->scanf("%f", &l_eye_l_y);
		real l_eye_r_x, l_eye_r_y;
		file->scanf("%f", &l_eye_r_x);
		file->scanf("%f", &l_eye_r_y);
		
		ldm_points[n_ldm_points].x = (l_eye_l_x + l_eye_r_x) / 2.0;
		ldm_points[n_ldm_points++].y = (l_eye_l_y + l_eye_r_y) / 2.0;

		real r_eye_l_x, r_eye_l_y;
		file->scanf("%f", &r_eye_l_x);
		file->scanf("%f", &r_eye_l_y);
		real r_eye_r_x, r_eye_r_y;
		file->scanf("%f", &r_eye_r_x);
		file->scanf("%f", &r_eye_r_y);

		ldm_points[n_ldm_points].x = (r_eye_l_x + r_eye_r_x) / 2.0;
		ldm_points[n_ldm_points++].y = (r_eye_l_y + r_eye_r_y) / 2.0;

		l_eye_idx = 0;
		r_eye_idx = 1;

		if(verbose) message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
	}
	else if(postype == 4)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 4 * sizeof(sPoint2D));

		if(verbose) message("reading eye corners + nose tip + chin pos format (non frontal 22.5 degree).");
	
		int n_;
		
		file->scanf("%d", &n_);

		if(n_ != 6) error("Number of points (%d) <> 6", n_);
		   
		// left eye
		real l_eye_l_x, l_eye_l_y;
		file->scanf("%f", &l_eye_l_x);
		file->scanf("%f", &l_eye_l_y);
		real l_eye_r_x, l_eye_r_y;
		file->scanf("%f", &l_eye_r_x);
		file->scanf("%f", &l_eye_r_y);
		
		ldm_points[n_ldm_points].x = (l_eye_l_x + l_eye_r_x) / 2.0;
		ldm_points[n_ldm_points++].y = (l_eye_l_y + l_eye_r_y) / 2.0;

		// right eye
		real r_eye_l_x, r_eye_l_y;
		file->scanf("%f", &r_eye_l_x);
		file->scanf("%f", &r_eye_l_y);
		real r_eye_r_x, r_eye_r_y;
		file->scanf("%f", &r_eye_r_x);
		file->scanf("%f", &r_eye_r_y);

		ldm_points[n_ldm_points].x = (r_eye_l_x + r_eye_r_x) / 2.0;
		ldm_points[n_ldm_points++].y = (r_eye_l_y + r_eye_r_y) / 2.0;

		// nose tip
		real x, y;
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		// chin
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		l_eye_idx = 0;
		r_eye_idx = 1;
		nosetip_idx = 2;
		chin_idx = 3;

		if(verbose)
		{
			message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
			message(" + Nosetip (%.1f-%.1f)", getNosetip().x, getNosetip().y);
			message(" + Chin (%.1f-%.1f)", getChin().x, getChin().y);
		}
	}
	else if(postype == 5)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 4 * sizeof(sPoint2D));

		if(verbose) message("reading left eye corners + right eye center + nose tip + chin pos format (non frontal 45/67.5 degree).");
		
		int n_;
		
		file->scanf("%d", &n_);

		if(n_ != 5) error("Number of points (%d) <> 5", n_);
		   
		// left eye
		real l_eye_l_x, l_eye_l_y;
		file->scanf("%f", &l_eye_l_x);
		file->scanf("%f", &l_eye_l_y);
		real l_eye_r_x, l_eye_r_y;
		file->scanf("%f", &l_eye_r_x);
		file->scanf("%f", &l_eye_r_y);
		
		ldm_points[n_ldm_points].x = (l_eye_l_x + l_eye_r_x) / 2.0;
		ldm_points[n_ldm_points++].y = (l_eye_l_y + l_eye_r_y) / 2.0;

		// right eye
		real x, y;
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		// nose tip
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		// chin
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		l_eye_idx = 0;
		r_eye_idx = 1;
		nosetip_idx = 2;
		chin_idx = 3;

		if(verbose)
		{
			message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
			message(" + Nosetip (%.1f-%.1f)", getNosetip().x, getNosetip().y);
			message(" + Chin (%.1f-%.1f)", getChin().x, getChin().y);
		}
	}
	else if(postype == 6)
	{
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, 4 * sizeof(sPoint2D));

		if(verbose) message("reading left eye center + nose tip + chin pos format  (non frontal 90 degree).");
		
		int n_;
		
		file->scanf("%d", &n_);

		if(n_ != 3) error("Number of points (%d) <> 3", n_);
		   
		// left eye
		real x, y;
		file->scanf("%f", &x);
		file->scanf("%f", &y);
		
		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		// nose tip
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		// chin
		file->scanf("%f", &x);
		file->scanf("%f", &y);

		ldm_points[n_ldm_points].x = x;
		ldm_points[n_ldm_points++].y = y;

		l_eye_idx = 0;
		nosetip_idx = 1;
		chin_idx = 2;

		if(verbose)
		{
			message(" + Left eye (%.1f-%.1f)", getLeye().x, getLeye().y);
			message(" + Nosetip (%.1f-%.1f)", getNosetip().x, getNosetip().y);
			message(" + Chin (%.1f-%.1f)", getChin().x, getChin().y);
		}
	}
	else if((postype >= 68) && (postype <= 71))
	{
		if(verbose) message("reading Tim Cootes Markup point format.");
		
		char buffer[250];
		do
		{
			file->gets(buffer, 250);
		} while(buffer[0] != '{');

		n_ldm_points = 68;
		ldm_points = (sPoint2D *)allocator->realloc((sPoint2D *) ldm_points, n_ldm_points * sizeof(sPoint2D));

		for(int i = 0 ; i < 68 ; i++)
		{
		   	real x, y;
			file->scanf("%g", &x);
			file->scanf("%g", &y);
			ldm_points[i].x = x;
			ldm_points[i].y = y;
		
			if(verbose) print("> %g %g\n", x, y);
			
			l_eye_idx = 31;
			r_eye_idx = 36;

		}
/*
		// This bit is specific to ipLocalFaceExtract
		if(postype == 69)
		{
			// Eyebrows corners, Eyes corners, Nose and Mouth

			int n_points_conf1 = 15;
			int points_conf1[] = {15, 18, 21, 24, 27, 29, 32, 45, 46, 47, 48, 51, 54, 57, 67};

			if(verbose)
			{
				print("Using sub-set of points:");
				for(int j = 0 ; j < n_points_conf1 ; j++) print(" %d", points_conf1[j]);
				print("\n");
			}

			n_ldm_points = 0;
			
			for(int i = 0 ; i < 68 ; i++)
			{
				// search i in points_conf1
				for(int j = 0 ; j < n_points_conf1 ; j++)
					if(points_conf1[j] == i)
					{
						ldm_points[n_ldm_points].x = ldm_points[i].x;
						ldm_points[n_ldm_points].y = ldm_points[i].y;

						n_ldm_points++;
					}
			}

			if(verbose)
			{
				for(int i = 0 ; i < n_ldm_points ; i++)
					print("> %g %g\n", ldm_points[i].x, ldm_points[i].y);
			}
		}
		else if(postype == 70)
		{
		   	//

			int n_points_conf2 = 5;
			int points_conf2[] = {31, 36, 51, 57, 67};

			if(verbose)
			{
				print("Using sub-set of points:");
				for(int j = 0 ; j < n_points_conf2 ; j++) print(" %d", points_conf2[j]);
				print("\n");
			}
			
			n_ldm_points = 0;
			
			for(int i = 0 ; i < 68 ; i++)
			{
				// search i in points_conf2
				for(int j = 0 ; j < n_points_conf2 ; j++)
					if(points_conf2[j] == i)
					{
						ldm_points[n_ldm_points].x = ldm_points[i].x;
						ldm_points[n_ldm_points].y = ldm_points[i].y;

						n_ldm_points++;
					}
			}

			if(verbose)
				for(int i = 0 ; i < n_points ; i++) print("> %g %g\n", ldm_points[i].x, ldm_points[i].y);
		}
		else if(postype == 71)
		{
		   	//

			int n_points_conf3 = 21;
			int points_conf3[] = {15, 18, 21, 24, 27, 31, 32, 36, 37, 38, 39, 40, 42, 43, 44, 45, 48, 54, 51, 57, 67};

			if(verbose)
			{
				print("Using sub-set of points:");
				for(int j = 0 ; j < n_points_conf3 ; j++) print(" %d", points_conf3[j]);
				print("\n");
			}
			
			n_ldm_points = 0;
			
			for(int i = 0 ; i < 68 ; i++)
			{
				// search i in points_conf3
				for(int j = 0 ; j < n_points_conf3 ; j++)
					if(points_conf3[j] == i)
					{
						ldm_points[n_ldm_points].x = ldm_points[i].x;
						ldm_points[n_ldm_points].y = ldm_points[i].y;

						n_ldm_points++;
					}
			}

			if(verbose)
				for(int i = 0 ; i < n_points ; i++) print("> %g %g\n", ldm_points[i].x, ldm_points[i].y);
		}
*/
		if(verbose) message(" + Eyes (%.1f-%.1f) - (%.1f-%.1f)", getLeye().x, getLeye().y, getReye().x, getReye().y);
	}
	else warning("FaceModel: Invalid postype (%d).", postype);
}

FaceModel::~FaceModel()
{
}

}

