#ifndef LDA_TRAINER_INC
#define LDA_TRAINER_INC

#include "Trainer.h"
#include "EigenTrainer.h"
#include "LDAMachine.h"

namespace Torch {

/** This class is designed to compute Linear Discriminant Analysis

    This class only computes the between-class scatter matrix and the within-class scatter matrix.
    
    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 1.0
*/
class LDATrainer : public Trainer
{
protected:
	//
	Mat *M_;

public:
   	/// number of inputs
	int n_inputs;

	/// number of classes
	int n_classes;
	
	/// between-class scatter matrix #Sb# of size #n_inputs#x#n_inputs#
	Mat *Sb;

	/// within-class scatter matrix #Sw# of size #n_inputs#x#n_inputs#
	Mat *Sw;

	/// means of each class
	real **mean_;
   	
	/// Mean of class means
	real *mean_class_mean_;

	/// projection of #mean_class_mean_# into LDA space
	real *d_m_mean_;
	
	/// eigenvalues in descending order of $S_w^{-1} * S_b$
    	Vec *eigenvalues;

	/// eigenvalues of the LDAMachine (to be filled)
	real *eigenvalues_;

	/// LDA basis vector
    	Mat *eigenvectors;

	/// LDA basis vector of the LDAMachine (to be filled)
	real **eigenvectors_;

	/// verbosity level
	int verbose_level;

	/// consider the target of an example as its class, otherwise attribute the class assuming the same number of example per class
	bool classprovided;

	/// consider the within scatter matrix as the identity matrix
	bool within_is_identity;

	/// save scatter matrices
	bool save;

	//-----
	
	///
	LDATrainer(int n_classes_, LDAMachine *machine_);

	/// compute between and within scatter matrices
	void makeSbSw(DataSet *data_);

	///
	virtual void train(DataSet *data_, MeasurerList *measurers_ = NULL) = 0;

	///
	virtual ~LDATrainer();
};

}

#endif
