#ifndef IP_FACE_EXTRACT_INC
#define IP_FACE_EXTRACT_INC

#include "ipCore.h"
#include "FaceModel.h"

namespace Torch {

/** This class is designed to extract faces from an image

    \begin{verbatim}
    Given an image and some optional parameters (shift, scale, ..), 
    this class "extracts" subwindows face samples. These subwindows
    are available in seq_out sequence. Before running the process()
    method, one must call setPos() that initializes the groundtruth
    eye positions.
    \end{verbatim}

    @author Yann Rodriguez (rodrig@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 1.0
*/
class ipFaceExtract : public ipCore
{
private:
	// small perturbations added to the groundtruth eye positions.
	int shift;
	int scale;

	// in-plane rotation
	int rotat;
	
	//
	real angle_offset;

	/* number of face samples randomly chosen among the set of all samples
	    generated from 1 face, according to shift, scale and rotat.
	*/
	int n_selected;

	// the face model
	FaceModel *face_model;

	int width_in;
	int height_in;

	int width_out;
	int height_out;
	
public:

	///
    	ipFaceExtract(int width_in_, int height_in_, int width_out_, int height_out_, const char *coding_, FaceModel *face_model_, real angle_offset_ = 0.0, int shift=0, int scale=0, int rotat=0, int n_selected=1);

	///
	virtual void process(Sequence *seq_in);

	///
	virtual int getWidthOut(void);

	///
	virtual int getHeightOut(void);
	
	///
	virtual ~ipFaceExtract();
};

}

#endif
