#ifndef FFMPEG_VIDEO_FILE_INC
#define FFMPEG_VIDEO_FILE_INC

//#define RAW_RGB_SUPPORT

#include "VideoFile.h"

#ifdef HAVE_FFMPEG
extern "C"
{
#include "avformat.h"
}
#endif

namespace Torch {

/** This class is designed to read/write video files using the FFmpeg library
	
    @author Sebastien Marcel (marcel@idiap.ch) 
    @author Julien Tiphaigne
    @version 2.0
    \Date
    @since 1.0
*/
class ffmpegVideoFile : public VideoFile
{
   	// number of instances of the class
	static int n_instance;
	
	//
	bool readMode;
	bool writeMode;

	//---
	
#ifdef HAVE_FFMPEG
	
	/*
	 
	 TYPE			: example	: ffmpeg struc name
 	 ----------------------------------------------------------------------
	 FILE			: xxx.avi	: ByteIOContext
	 FORMAT 		: AVI 		: AVFormatContext
	 STREAMS		: Video		: AVStream
	 CODEC 			: Divx		: AVCodecContext
	 COMPRESSED FRAME	: data		: AVPacket
	 UNCOMPRESSED FRAME	: picture	: AVPicture or AVFrame (similar)
	
	See avformat.h and avcodec.h for more details
	*/

	// Struct that contains input format parameters (avformat.h)
	AVFormatContext *ic;

	// Struct that contains ouput format parameters (avformat.h)
	AVFormatContext *oc;

	// Struct that contains format fonctions names  (avformat.h)
	AVInputFormat *iformat;

	// Struct that contains streams parameters  (avformat.h)
    	AVStream *st;

	// Struct with a pointer of a read frame  (avformat.h)
	AVPacket packt, *pkt;

	// True if input file is RAW RGB file
	bool no_codec;

	//---
	
	// Struct contains codec parameters (avcodec.h)
	AVCodecContext *c;

	// Struct contains codec fonctions names (avcodec.h)
	AVCodec *codec;

	// uncompressed frame (avcodec.h)
	AVFrame *avframe;
    	char buf[1024];
	
	// a picture (like a AVFrame) (avcodec.h)
	AVPicture pic1, *pict;
	
	// Struct that contains file parameters
       	ByteIOContext pb1, *pb;
	
	// true if seeking (important for next read())
	bool seek;
	int begin, outbuf_size;
	uint8_t *outbuf;
#endif

	// the bitrate to use during encoding
	int bitrate;
	
	// the size of Group Of Picture (max interval between keyframes)
	int gop;
public:
	/** @name constructors */
	//@{
	///
  	ffmpegVideoFile();

	///
	ffmpegVideoFile(char *filename_, const char *open_flags_ = "r");
	//@}
	
	///
  	virtual ~ffmpegVideoFile();

	/// open the video file #filename#
	virtual bool open(char *filename_, const char *open_flags_ = "r");

	/// read the current image from the file and fill #pixmap#
	virtual bool read();

	/// write #pixmap_# into a file
	virtual bool write(unsigned char *pixmap_);

	/// restart the video file at the beginning
	virtual bool rewindmovie();

	/// seek in the video file
	virtual int seekmovie(int h, int m, int s, int f);
	
	/// close the video file
  	virtual bool close();
};

}

#endif
